@@ -23,17 +23,16 @@ import webbrowser
 
 import jsonschema
 from oslo_utils import uuidutils
+import six
 import yaml
 
 from rally import api
 from rally.cli import cliutils
 from rally.cli import envutils
-from rally.common import db
 from rally.common import fileutils
 from rally.common.i18n import _
 from rally.common import junit
-from rally.common import log as logging
-from rally.common import objects
+from rally.common import logging
 from rally.common import utils as rutils
 from rally import consts
 from rally import exceptions
@@ -117,13 +116,28 @@ class TaskCommands(object):
 
             print(_("Input task is:\n%s\n") % rendered_task)
             try:
-                return yaml.safe_load(rendered_task)
+                parsed_task = yaml.safe_load(rendered_task)
+
             except Exception as e:
                 print(_("Wrong format of rendered input task. It should be "
                         "YAML or JSON.\n%s") % e,
                       file=sys.stderr)
                 raise FailedToLoadTask()
 
+            print(_("Task syntax is correct :)"))
+            return parsed_task
+
+    def _load_and_validate_task(self, task, task_args, task_args_file,
+                                deployment, task_instance=None):
+        if not os.path.exists(task) or os.path.isdir(task):
+            if task_instance:
+                task_instance.set_failed(log="No such file '%s'" % task)
+            raise IOError("File '%s' is not found." % task)
+        input_task = self._load_task(task, task_args, task_args_file)
+        api.Task.validate(deployment, input_task, task_instance)
+        print(_("Task config is valid :)"))
+        return input_task
+
     @cliutils.args("--deployment", type=str, dest="deployment",
                    required=False, help="UUID or name of the deployment")
     @cliutils.args("--task", "--filename",
@@ -153,16 +167,11 @@ class TaskCommands(object):
         :param deployment: UUID or name of a deployment
         """
         try:
-            input_task = self._load_task(task, task_args, task_args_file)
-        except FailedToLoadTask:
-            return(1)
+            self._load_and_validate_task(task, task_args, task_args_file,
+                                         deployment)
 
-        try:
-            api.Task.validate(deployment, input_task)
-            print("Task config is valid :)")
-        except exceptions.InvalidTaskException as e:
-            print("Task config is invalid: \n")
-            print(e)
+        except (exceptions.InvalidTaskException, FailedToLoadTask) as e:
+            print(e, file=sys.stderr)
             return(1)
 
     @cliutils.args("--deployment", type=str, dest="deployment",
@@ -202,25 +211,32 @@ class TaskCommands(object):
                                      scenario will stop when any SLA check
                                      for it fails
         """
-        try:
-            input_task = self._load_task(task, task_args, task_args_file)
-        except FailedToLoadTask:
-            return(1)
+
+        task_instance = api.Task.create(deployment, tag)
 
         try:
-            task = api.Task.create(deployment, tag)
+            input_task = self._load_and_validate_task(
+                task, task_args, task_args_file, deployment,
+                task_instance=task_instance)
+
             print(cliutils.make_header(
                   _("Task %(tag)s %(uuid)s: started")
-                  % {"uuid": task["uuid"], "tag": task["tag"]}))
+                  % {"uuid": task_instance["uuid"],
+                     "tag": task_instance["tag"]}))
             print("Benchmarking... This can take a while...\n")
             print("To track task status use:\n")
             print("\trally task status\n\tor\n\trally task detailed\n")
+
             if do_use:
-                self.use(task["uuid"])
-            api.Task.start(deployment, input_task, task=task,
+                self.use(task_instance["uuid"])
+
+            api.Task.start(deployment, input_task, task=task_instance,
                            abort_on_sla_failure=abort_on_sla_failure)
-            self.detailed(task_id=task["uuid"])
-        except exceptions.InvalidConfigException:
+            self.detailed(task_id=task_instance["uuid"])
+
+        except (exceptions.InvalidTaskException, FailedToLoadTask) as e:
+            task_instance.set_failed(log=e.format_message())
+            print(e, file=sys.stderr)
             return(1)
 
     @cliutils.args("--uuid", type=str, dest="task_id", help="UUID of task")
@@ -253,7 +269,7 @@ class TaskCommands(object):
         Returns current status of task
         """
 
-        task = db.task_get(task_id)
+        task = api.Task.get(task_id)
         print(_("Task %(task_id)s: %(status)s")
               % {"task_id": task_id, "status": task["status"]})
 
@@ -272,7 +288,8 @@ class TaskCommands(object):
         Prints detailed information of task.
         """
 
-        def _print_iterations_data(raw_data):
+        def _print_iterations_data(result):
+            raw_data = result["data"]["raw"]
             headers = ["iteration", "full duration"]
             float_cols = ["full duration"]
             atomic_actions = []
@@ -297,38 +314,36 @@ class TaskCommands(object):
                 if r["atomic_actions"]:
                     for action in atomic_actions:
                         dlist.append(r["atomic_actions"].get(action) or 0)
-                table_rows.append(rutils.Struct(**dict(zip(headers, dlist))))
+                table_rows.append(rutils.Struct(**dict(zip(headers,
+                                                           dlist))))
             cliutils.print_list(table_rows,
                                 fields=headers,
                                 formatters=formatters)
             print()
 
-        task = db.task_get_detailed(task_id)
-
-        if task is None:
-            print("The task %s can not be found" % task_id)
-            return(1)
-
-        print()
-        print("-" * 80)
-        print(_("Task %(task_id)s: %(status)s")
-              % {"task_id": task_id, "status": task["status"]})
-
-        if task["status"] == consts.TaskStatus.FAILED:
+        def _print_task_info(task):
+            print()
             print("-" * 80)
-            verification = yaml.safe_load(task["verification_log"])
+            print(_("Task %(task_id)s: %(status)s")
+                  % {"task_id": task_id, "status": task["status"]})
 
-            if not logging.is_debug():
-                print(verification[0])
-                print(verification[1])
-                print()
-                print(_("For more details run:\nrally -vd task detailed %s")
-                      % task["uuid"])
-            else:
-                print(yaml.safe_load(verification[2]))
-            return
+            if task["status"] == consts.TaskStatus.FAILED:
+                print("-" * 80)
+                verification = yaml.safe_load(task["verification_log"])
 
-        for result in task["results"]:
+                if not logging.is_debug():
+                    print(verification[0])
+                    print(verification[1])
+                    print()
+                    print(_("For more details run:\n"
+                            "rally -vd task detailed %s")
+                          % task["uuid"])
+                else:
+                    print(yaml.safe_load(verification[2]))
+                return False
+            return True
+
+        def _print_scenario_args(result):
             key = result["key"]
             print("-" * 80)
             print()
@@ -337,6 +352,7 @@ class TaskCommands(object):
             print("args values:")
             print(json.dumps(key["kw"], indent=2))
 
+        def _print_summrized_result(result):
             raw = result["data"]["raw"]
             table_cols = ["action", "min", "median",
                           "90%ile", "95%ile", "max",
@@ -365,19 +381,16 @@ class TaskCommands(object):
                 else:
                     data = [action, None, None, None, None, None, None,
                             "0.0%", len(raw)]
-                table_rows.append(rutils.Struct(**dict(zip(table_cols, data))))
+                table_rows.append(rutils.Struct(**dict(zip(table_cols,
+                                                           data))))
 
             cliutils.print_list(table_rows, fields=table_cols,
                                 formatters=formatters,
                                 table_label="Response Times (sec)",
                                 sortby_index=None)
 
-            if iterations_data:
-                _print_iterations_data(raw)
-
-            print(_("Load duration: %s") % result["data"]["load_duration"])
-            print(_("Full duration: %s") % result["data"]["full_duration"])
-
+        def _print_ssrs_result(result):
+            raw = result["data"]["raw"]
             # NOTE(hughsaunders): ssrs=scenario specific results
             ssrs = []
             for result in raw:
@@ -410,7 +423,8 @@ class TaskCommands(object):
                                round(utils.mean(values), 3)]
                     else:
                         row = [str(key)] + ["n/a"] * 6
-                    table_rows.append(rutils.Struct(**dict(zip(headers, row))))
+                    table_rows.append(rutils.Struct(**dict(zip(headers,
+                                                               row))))
                 print("\nScenario Specific Results\n")
                 cliutils.print_list(table_rows,
                                     fields=headers,
@@ -422,17 +436,37 @@ class TaskCommands(object):
                     if errors:
                         print(errors)
 
-        print()
-        print("HINTS:")
-        print(_("* To plot HTML graphics with this data, run:"))
-        print("\trally task report %s --out output.html" % task["uuid"])
-        print()
-        print(_("* To generate a JUnit report, run:"))
-        print("\trally task report %s --junit --out output.xml" %
-              task["uuid"])
-        print()
-        print(_("* To get raw JSON output of task results, run:"))
-        print("\trally task results %s\n" % task["uuid"])
+        def _print_hints(task):
+            print()
+            print("HINTS:")
+            print(_("* To plot HTML graphics with this data, run:"))
+            print("\trally task report %s --out output.html" % task["uuid"])
+            print()
+            print(_("* To generate a JUnit report, run:"))
+            print("\trally task report %s --junit --out output.xml" %
+                  task["uuid"])
+            print()
+            print(_("* To get raw JSON output of task results, run:"))
+            print("\trally task results %s\n" % task["uuid"])
+
+        task = api.Task.get_detailed(task_id)
+
+        if task is None:
+            print("The task %s can not be found" % task_id)
+            return(1)
+
+        if _print_task_info(task):
+            for result in task["results"]:
+                _print_scenario_args(result)
+                _print_summrized_result(result)
+                if iterations_data:
+                    _print_iterations_data(result)
+                _print_ssrs_result(result)
+                print(_("Load duration: %s") %
+                      result["data"]["load_duration"])
+                print(_("Full duration: %s") %
+                      result["data"]["full_duration"])
+            _print_hints(task)
 
     @cliutils.args("--uuid", type=str, dest="task_id", help="uuid of task")
     @envutils.with_default_task_id
@@ -448,13 +482,14 @@ class TaskCommands(object):
                     "sla": x["data"]["sla"],
                     "load_duration": x["data"]["load_duration"],
                     "full_duration": x["data"]["full_duration"]}
-                   for x in objects.Task.get(task_id).get_results()]
+                   for x in api.Task.get(task_id).get_results()]
 
         if results:
             print(json.dumps(results, sort_keys=True, indent=4))
         else:
-            print(_("The task %s is still running, results will become "
-                    "available when it is finished.") % task_id)
+            print(_("The task %s marked as '%s'. Results "
+                    "available when it is '%s' .") % (
+                task_id, consts.TaskStatus.FAILED, consts.TaskStatus.FINISHED))
             return(1)
 
     @cliutils.args("--deployment", type=str, dest="deployment",
@@ -499,7 +534,7 @@ class TaskCommands(object):
         if not all_deployments:
             filters.setdefault("deployment", deployment)
 
-        task_list = [task.to_dict() for task in objects.Task.list(**filters)]
+        task_list = [task.to_dict() for task in api.Task.list(**filters)]
 
         for x in task_list:
             x["duration"] = x["updated_at"] - x["created_at"]
@@ -531,10 +566,15 @@ class TaskCommands(object):
     @cliutils.args("--html", dest="out_format",
                    action="store_const", const="html",
                    help="Generate the report in HTML.")
+    @cliutils.args("--html-static", dest="out_format",
+                   action="store_const", const="html_static",
+                   help=("Generate the report in HTML with embedded "
+                         "JS and CSS, so it will not depend on "
+                         "the Internet availability."))
     @cliutils.args("--junit", dest="out_format",
                    action="store_const", const="junit",
                    help="Generate the report in the JUnit format.")
-    @envutils.default_from_global("tasks", envutils.ENV_TASK, "--uuid")
+    @envutils.default_from_global("tasks", envutils.ENV_TASK, "uuid")
     @cliutils.suppress_warnings
     def report(self, tasks=None, out=None, open_it=False, out_format="html"):
         """Generate report file for specified task.
@@ -543,7 +583,7 @@ class TaskCommands(object):
         :param tasks: list, UUIDs od tasks or pathes files with tasks results
         :param out: str, output file name
         :param open_it: bool, whether to open output file in web browser
-        :param out_format: output format (junit or html)
+        :param out_format: output format (junit, html or html_static)
         """
 
         tasks = isinstance(tasks, list) and tasks or [tasks]
@@ -560,14 +600,11 @@ class TaskCommands(object):
                         try:
                             jsonschema.validate(
                                 result,
-                                objects.task.TASK_RESULT_SCHEMA)
+                                api.Task.TASK_RESULT_SCHEMA)
                         except jsonschema.ValidationError as e:
                             print(_("ERROR: Invalid task result format in %s")
                                   % task_file_or_uuid, file=sys.stderr)
-                            if logging.is_debug():
-                                print(e, file=sys.stderr)
-                            else:
-                                print(e.message, file=sys.stderr)
+                            print(six.text_type(e), file=sys.stderr)
                             return 1
 
             elif uuidutils.is_uuid_like(task_file_or_uuid):
@@ -577,7 +614,7 @@ class TaskCommands(object):
                                "result": x["data"]["raw"],
                                "load_duration": x["data"]["load_duration"],
                                "full_duration": x["data"]["full_duration"]},
-                    objects.Task.get(task_file_or_uuid).get_results())
+                    api.Task.get(task_file_or_uuid).get_results())
             else:
                 print(_("ERROR: Invalid UUID or file name passed: %s"
                         ) % task_file_or_uuid,
@@ -595,9 +632,10 @@ class TaskCommands(object):
 
         output_file = os.path.expanduser(out)
 
-        if out_format == "html":
+        if out_format.startswith("html"):
             with open(output_file, "w+") as f:
-                f.write(plot.plot(results))
+                f.write(plot.plot(results,
+                                  include_libs=(out_format == "html_static")))
             if open_it:
                 webbrowser.open_new_tab("file://" + os.path.realpath(out))
         elif out_format == "junit":
@@ -657,7 +695,7 @@ class TaskCommands(object):
         :param task_id: Task uuid.
         :returns: Number of failed criteria.
         """
-        results = objects.Task.get(task_id).get_results()
+        results = api.Task.get(task_id).get_results()
         failed_criteria = 0
         data = []
         STATUS_PASS = "PASS"
@@ -687,5 +725,5 @@ class TaskCommands(object):
         :param task: Task uuid.
         """
         print("Using task: %s" % task)
-        db.task_get(task)
+        api.Task.get(task)
         fileutils.update_globals_file("RALLY_TASK", task)
