@@ -24,77 +24,135 @@ import six
 from rally import api
 from rally.cli import cliutils
 from rally.cli import envutils
-from rally.common import db
 from rally.common import fileutils
 from rally.common.i18n import _
-from rally.common import objects
+from rally.common import utils
 from rally import consts
 from rally import exceptions
 from rally.verification.tempest import diff
 from rally.verification.tempest import json2html
 
 
-class VerifyCommands(object):
-    """Test cloud with Tempest
+AVAILABLE_SETS = list(consts.TempestTestsSets) + list(consts.TempestTestsAPI)
 
-    Set of commands that allow you to perform Tempest tests of
-    OpenStack live cloud.
+
+class VerifyCommands(object):
+    """Verify an OpenStack cloud via Tempest.
+
+    Set of commands that allow you to run Tempest tests.
     """
 
     @cliutils.args("--deployment", dest="deployment", type=str,
-                   required=False, help="UUID or name of a deployment.")
+                   required=False, help="UUID or name of a deployment")
     @cliutils.args("--set", dest="set_name", type=str, required=False,
-                   help="Name of tempest test set. Available sets: %s" % ", ".
-                   join(list(consts.TempestTestsSets) +
-                        list(consts.TempestTestsAPI)))
+                   help="Name of a Tempest test set. "
+                        "Available sets are %s" % ", ".join(AVAILABLE_SETS))
     @cliutils.args("--regex", dest="regex", type=str, required=False,
-                   help="Regular expression of test.")
+                   help="Regular expression of test")
+    @cliutils.args("--tests-file", dest="tests_file", type=str,
+                   help="Path to a file with a list of Tempest tests",
+                   required=False)
     @cliutils.args("--tempest-config", dest="tempest_config", type=str,
                    required=False,
                    help="User specified Tempest config file location")
     @cliutils.args("--no-use", action="store_false", dest="do_use",
                    help="Don't set new task as default for future operations")
     @cliutils.args("--system-wide-install", dest="system_wide_install",
-                   help="Use virtualenv else run tests in local environment",
+                   help="Don't create a virtual env when installing Tempest; "
+                        "use the local env instead of the Tempest virtual env "
+                        "when running the tests. Take notice that all Tempest "
+                        "requirements have to be already installed in "
+                        "the local env!",
                    required=False, action="store_true")
+    @cliutils.args("--concurrency", dest="concur", type=int, required=False,
+                   help="How many processes to use to run Tempest tests. "
+                        "The default value (0) auto-detects your CPU count")
     @envutils.with_default_deployment(cli_arg_name="deployment")
-    def start(self, set_name="", deployment=None, regex=None,
-              tempest_config=None, do_use=True,
-              system_wide_install=False):
-        """Start set of tests.
+    def start(self, deployment=None, set_name="", regex=None,
+              tests_file=None, tempest_config=None, do_use=True,
+              system_wide_install=False, concur=0):
+        """Start verification (run Tempest tests).
 
-        :param set_name: Name of tempest test set
         :param deployment: UUID or name of a deployment
+        :param set_name: Name of a Tempest test set
         :param regex: Regular expression of test
+        :param tests_file: Path to a file with a list of Tempest tests
         :param tempest_config: User specified Tempest config file location
         :param do_use: Use new task as default for future operations
-        :param system_wide_install: Use virtualenv else run tests in
-                                    local environment
+        :param system_wide_install: Whether or not to create a virtual env
+                                    when installing Tempest; whether or not to
+                                    use the local env instead of the Tempest
+                                    virtual env when running the tests
+        :param concur: How many processes to use to run Tempest tests.
+                       The default value (0) auto-detects CPU count
         """
-
+        msg = _("Arguments '%s' and '%s' are not compatible. "
+                "You can use only one of the mentioned arguments.")
         if regex and set_name:
-            raise exceptions.InvalidArgumentsException("set_name and regex "
-                                                       "are not compatible")
-        if not (regex or set_name):
+            print(msg % ("regex", "set"))
+            return 1
+        if tests_file and set_name:
+            print(msg % ("tests_file", "set"))
+            return 1
+        if tests_file and regex:
+            print(msg % ("tests_file", "regex"))
+            return 1
+
+        if not (regex or set_name or tests_file):
             set_name = "full"
-        if set_name and set_name not in (list(consts.TempestTestsSets) +
-                                         list(consts.TempestTestsAPI)):
-            print("Sorry, but there are no desired tempest test set. Please "
-                  "choose from: %s" % ", ".join(list(consts.TempestTestsSets) +
-                                                list(consts.TempestTestsAPI)))
-            return (1)
-        verification = api.Verification.verify(deployment, set_name, regex,
-                                               tempest_config,
-                                               system_wide_install)
+
+        if set_name and set_name not in AVAILABLE_SETS:
+            print(_("Tempest test set '%s' not found "
+                    "in available test sets. Available sets are %s.")
+                  % (set_name, ", ".join(AVAILABLE_SETS)))
+            return 1
+
+        if tests_file and not os.path.exists(tests_file):
+            print(_("File '%s' not found.") % tests_file)
+            return 1
+
+        verification = api.Verification.verify(
+            deployment, set_name=set_name, regex=regex,
+            tests_file=tests_file, tempest_config=tempest_config,
+            system_wide_install=system_wide_install, concur=concur)
+        if do_use:
+            self.use(verification["uuid"])
+
+    @cliutils.args("--deployment", dest="deployment", type=str,
+                   required=False, help="UUID or name of a deployment")
+    @cliutils.args("--set", dest="set_name", type=str, required=False,
+                   help="Name of a Tempest test set. "
+                        "Available sets are %s" % ", ".join(AVAILABLE_SETS))
+    @cliutils.args("--file", dest="log_file", type=str,
+                   required=True,
+                   help="User specified Tempest log file location. "
+                        "Note, Tempest log file needs to be in subunit format")
+    @cliutils.args("--no-use", action="store_false", dest="do_use",
+                   required=False,
+                   help="Don't set new task as default for future operations")
+    @cliutils.alias("import")
+    @envutils.with_default_deployment(cli_arg_name="deployment")
+    def import_results(self, deployment=None, set_name="", log_file=None,
+                       do_use=True):
+        """Import Tempest tests results into the Rally database.
+
+        :param deployment: UUID or name of a deployment
+        :param set_name: Name of a Tempest test set
+        :param do_use: Use new task as default for future operations
+        :param log_file: User specified Tempest log file in subunit format
+        """
+        deployment, verification = api.Verification.import_results(deployment,
+                                                                   set_name,
+                                                                   log_file)
         if do_use:
             self.use(verification["uuid"])
 
     def list(self):
-        """Display all verifications table, started and finished."""
+        """Display verifications table."""
 
         fields = ["UUID", "Deployment UUID", "Set name", "Tests", "Failures",
                   "Created at", "Duration", "Status"]
-        verifications = db.verification_list()
+        verifications = api.Verification.list()
 
         for el in verifications:
             el["duration"] = el["updated_at"] - el["created_at"]
@@ -103,41 +161,41 @@ class VerifyCommands(object):
             cliutils.print_list(verifications, fields,
                                 sortby_index=fields.index("Created at"))
         else:
-            print(_("There are no results from verifier. To run a verifier, "
-                    "use:\nrally verify start"))
+            print(_("No verification was started yet. "
+                    "To start verification use:\nrally verify start"))
 
     @cliutils.args("--uuid", type=str, dest="verification_uuid",
-                   help="UUID of the verification")
+                   help="UUID of a verification")
     @cliutils.args("--html", action="store_true", dest="output_html",
-                   help=("Results will be in html format"))
+                   help="Display results in HTML format")
     @cliutils.args("--json", action="store_true", dest="output_json",
-                   help=("Results will be in json format"))
+                   help="Display results in JSON format")
     @cliutils.args("--output-file", type=str, required=False,
                    dest="output_file",
-                   help="If specified, output will be saved to given file")
+                   help="Path to a file to save results")
     @envutils.with_default_verification_id
     @cliutils.suppress_warnings
     def results(self, verification_uuid=None, output_file=None,
                 output_html=None, output_json=None):
-        """Get raw results of the verification.
+        """Display results of a verification.
 
-        :param verification_uuid: Verification UUID
-        :param output_file: If specified, output will be saved to given file
-        :param output_html: The output will be in HTML format
-        :param output_json: The output will be in JSON format (Default)
+        :param verification_uuid: UUID of a verification
+        :param output_file: Path to a file to save results
+        :param output_html: Display results in HTML format
+        :param output_json: Display results in JSON format (Default)
         """
-
         try:
-            results = db.verification_result_get(verification_uuid)["data"]
+            results = api.Verification.get(verification_uuid).get_results()
         except exceptions.NotFoundException as e:
             print(six.text_type(e))
             return 1
 
         result = ""
         if output_json + output_html > 1:
-            print("Please specify only one output format.")
+            print(_("Please specify only one "
+                    "output format: --json or --html."))
         elif output_html:
-            result = json2html.HtmlOutput(results).create_report()
+            result = json2html.generate_report(results)
         else:
             result = json.dumps(results, sort_keys=True, indent=4)
 
@@ -152,77 +210,80 @@ class VerifyCommands(object):
                    required=False,
                    help="UUID of a verification")
     @cliutils.args("--sort-by", dest="sort_by", type=str, required=False,
-                   help="Tests can be sorted by 'name' or 'duration'")
+                   help="Sort results by 'name' or 'duration'")
     @cliutils.args("--detailed", dest="detailed", action="store_true",
-                   required=False, help="Prints traceback of failed tests")
+                   required=False,
+                   help="Display detailed errors of failed tests")
     @envutils.with_default_verification_id
     def show(self, verification_uuid=None, sort_by="name", detailed=False):
-        """Display results table of the verification."""
+        """Display results table of a verification.
 
+        :param verification_uuid: UUID of a verification
+        :param sort_by: Sort results by 'name' or 'duration'
+        :param detailed: Display detailed errors of failed tests
+        """
         try:
             sortby_index = ("name", "duration").index(sort_by)
         except ValueError:
-            print("Sorry, but verification results can't be sorted "
-                  "by '%s'." % sort_by)
+            print(_("Verification results can't be sorted by '%s'.") % sort_by)
             return 1
 
         try:
-            verification = db.verification_get(verification_uuid)
-            tests = db.verification_result_get(verification_uuid)
+            verification = api.Verification.get(verification_uuid)
+            tests = verification.get_results()
         except exceptions.NotFoundException as e:
             print(six.text_type(e))
             return 1
 
-        print ("Total results of verification:\n")
+        print(_("Total results of verification:\n"))
         total_fields = ["UUID", "Deployment UUID", "Set name", "Tests",
                         "Failures", "Created at", "Status"]
         cliutils.print_list([verification], fields=total_fields)
 
-        print ("\nTests:\n")
+        print(_("\nTests:\n"))
         fields = ["name", "time", "status"]
 
-        values = [objects.Verification(test)
-                  for test in six.itervalues(tests.data["test_cases"])]
+        results = tests["test_cases"]
+        values = [utils.Struct(**results[test_name]) for test_name in results]
         cliutils.print_list(values, fields, sortby_index=sortby_index)
 
         if detailed:
-            for test in six.itervalues(tests.data["test_cases"]):
-                if test["status"] == "FAIL":
+            for test in six.itervalues(tests["test_cases"]):
+                if test["status"] == "fail":
                     header = cliutils.make_header(
                         "FAIL: %(name)s\n"
-                        "Time: %(time)s\n"
-                        "Type: %(type)s" % {"name": test["name"],
-                                            "time": test["time"],
-                                            "type": test["failure"]["type"]})
+                        "Time: %(time)s" % {"name": test["name"],
+                                            "time": test["time"]})
                     formatted_test = "%(header)s%(log)s\n" % {
                         "header": header,
-                        "log": test["failure"]["log"]}
-                    print (formatted_test)
+                        "log": test["traceback"]}
+                    print(formatted_test)
 
     @cliutils.args("--uuid", dest="verification_uuid", type=str,
-                   required=False,
-                   help="UUID of a verification")
+                   required=False, help="UUID of a verification")
     @cliutils.args("--sort-by", dest="sort_by", type=str, required=False,
-                   help="Tests can be sorted by 'name' or 'duration'")
+                   help="Sort results by 'name' or 'duration'")
     @envutils.with_default_verification_id
     def detailed(self, verification_uuid=None, sort_by="name"):
-        """Display results table of verification with detailed errors."""
+        """Display results table of a verification with detailed errors.
 
+        :param verification_uuid: UUID of a verification
+        :param sort_by: Sort results by 'name' or 'duration'
+        """
         self.show(verification_uuid, sort_by, True)
 
-    @cliutils.args("--uuid-1", type=str, dest="uuid1",
+    @cliutils.args("--uuid-1", type=str, required=True, dest="uuid1",
                    help="UUID of the first verification")
-    @cliutils.args("--uuid-2", type=str, dest="uuid2",
+    @cliutils.args("--uuid-2", type=str, required=True, dest="uuid2",
                    help="UUID of the second verification")
     @cliutils.args("--csv", action="store_true", dest="output_csv",
-                   help=("Save results in csv format to specified file"))
+                   help="Display results in CSV format")
     @cliutils.args("--html", action="store_true", dest="output_html",
-                   help=("Save results in html format to specified file"))
+                   help="Display results in HTML format")
     @cliutils.args("--json", action="store_true", dest="output_json",
-                   help=("Save results in json format to specified file"))
+                   help="Display results in JSON format")
     @cliutils.args("--output-file", type=str, required=False,
-                   dest="output_file",
-                   help="If specified, output will be saved to given file")
+                   dest="output_file", help="Path to a file to save results")
     @cliutils.args("--threshold", type=int, required=False,
                    dest="threshold", default=0,
                    help="If specified, timing differences must exceed this "
@@ -232,28 +293,26 @@ class VerifyCommands(object):
                 output_json=None, threshold=0):
         """Compare two verification results.
 
-        :param uuid1: First Verification UUID
-        :param uuid2: Second Verification UUID
-        :param output_file: If specified, output will be saved to given file
-        :param output_csv: Save results in csv format to the specified file
-        :param output_html: Save results in html format to the specified file
-        :param output_json: Save results in json format to the specified file
-                            (Default)
+        :param uuid1: UUID of the first verification
+        :param uuid2: UUID of the second verification
+        :param output_file: Path to a file to save results
+        :param output_csv: Display results in CSV format
+        :param output_html: Display results in HTML format
+        :param output_json: Display results in JSON format (Default)
         :param threshold: Timing difference threshold percentage
         """
-
         try:
-            results1 = db.verification_result_get(uuid1)["data"]["test_cases"]
-            results2 = db.verification_result_get(uuid2)["data"]["test_cases"]
-            _diff = diff.Diff(results1, results2, threshold)
+            res_1 = api.Verification.get(uuid1).get_results()["test_cases"]
+            res_2 = api.Verification.get(uuid2).get_results()["test_cases"]
+            _diff = diff.Diff(res_1, res_2, threshold)
         except exceptions.NotFoundException as e:
             print(six.text_type(e))
             return 1
 
         result = ""
         if output_json + output_html + output_csv > 1:
-            print("Please specify only one output format, either --json, "
-                  "--html or --csv.")
+            print(_("Please specify only one output "
+                    "format: --json, --html or --csv."))
             return 1
         elif output_html:
             result = _diff.to_html()
@@ -273,14 +332,14 @@ class VerifyCommands(object):
             print(result)
 
     @cliutils.args("--verification", type=str, dest="verification",
-                   required=False, help="UUID of the verification")
+                   required=False, help="UUID of a verification")
     def use(self, verification):
         """Set active verification.
 
-        :param verification: a UUID of verification
+        :param verification: UUID of a verification
         """
-        print("Verification UUID: %s" % verification)
-        db.verification_get(verification)
+        print(_("Verification UUID: %s") % verification)
+        api.Verification.get(verification)
         fileutils.update_globals_file("RALLY_VERIFICATION", verification)
 
     @cliutils.args("--deployment", dest="deployment", type=str,
@@ -297,7 +356,8 @@ class VerifyCommands(object):
 
         :param deployment: UUID or name of a deployment
         :param tempest_config: User specified Tempest config file location
-        :param override: Whether or not override existing Tempest config file
+        :param override: Whether or not to override existing Tempest
+                         config file
         """
         api.Verification.configure_tempest(deployment, tempest_config,
                                            override)
@@ -305,15 +365,21 @@ class VerifyCommands(object):
     @cliutils.args("--deployment", type=str, dest="deployment",
                    required=False, help="UUID or name of a deployment")
     @cliutils.args("--source", type=str, dest="source", required=False,
-                   help="Path/URL to repo to pull Tempest from")
+                   help="Path/URL to repo to clone Tempest from")
+    @cliutils.args("--no-tempest-venv", dest="no_tempest_venv",
+                   help="Don't create a virtual env for Tempest. Take notice "
+                        "that all Tempest requirements have to be already "
+                        "installed in the local env!",
+                   required=False, action="store_true")
     @envutils.with_default_deployment(cli_arg_name="deployment")
-    def install(self, deployment=None, source=None):
+    def install(self, deployment=None, source=None, no_tempest_venv=False):
         """Install Tempest.
 
         :param deployment: UUID or name of a deployment
-        :param source: Path/URL to repo to pull Tempest from
+        :param source: Path/URL to repo to clone Tempest from
+        :param no_tempest_venv: Whether or not to create a Tempest virtual env
         """
-        api.Verification.install_tempest(deployment, source)
+        api.Verification.install_tempest(deployment, source, no_tempest_venv)
 
     @cliutils.args("--deployment", type=str, dest="deployment",
                    required=False, help="UUID or name of a deployment")
@@ -330,15 +396,34 @@ class VerifyCommands(object):
     @cliutils.args("--tempest-config", dest="tempest_config", type=str,
                    required=False,
                    help="User specified Tempest config file location")
-    @cliutils.args("--source", type=str, dest="source",
-                   required=False, help="New path/URL to repo to pull Tempest "
-                                        "from. Use old source as default")
+    @cliutils.args("--source", type=str, dest="source", required=False,
+                   help="Path/URL to repo to clone Tempest from")
+    @cliutils.args("--no-tempest-venv", dest="no_tempest_venv",
+                   help="Don't create a virtual env for Tempest. Take notice "
+                        "that all Tempest requirements have to be already "
+                        "installed in the local env!",
+                   required=False, action="store_true")
     @envutils.with_default_deployment(cli_arg_name="deployment")
-    def reinstall(self, deployment=None, tempest_config=None, source=None):
-        """Uninstall Tempest and then reinstall from new source.
+    def reinstall(self, deployment=None,
+                  tempest_config=None, source=None, no_tempest_venv=False):
+        """Uninstall Tempest and install again.
 
         :param deployment: UUID or name of a deployment
         :param tempest_config: User specified Tempest config file location
-        :param source: New path/URL to repo to pull Tempest from
+        :param source: Path/URL to repo to clone Tempest from
+        :param no_tempest_venv: Whether or not to create a Tempest virtual env
         """
-        api.Verification.reinstall_tempest(deployment, tempest_config, source)
+        api.Verification.reinstall_tempest(deployment, tempest_config,
+                                           source, no_tempest_venv)
+
+    @cliutils.args("--deployment", type=str, dest="deployment",
+                   required=False, help="UUID or name of a deployment")
+    @envutils.with_default_deployment(cli_arg_name="deployment")
+    def showconfig(self, deployment=None):
+        """Show configuration file of Tempest.
+
+        :param deployment: UUID or name of a deployment
+        """
+        conf_info = api.Verification.show_config_info(deployment)
+        print(_("Tempest config file: %s") % conf_info["conf_path"])
+        print("\n" + conf_info["conf_data"])
