@@ -239,10 +239,7 @@ class HistogramChartTestCase(test.TestCase):
                       "key": "bar",
                       "values": [{"x": 2.2, "y": 1}, {"x": 3.2, "y": 1},
                                  {"x": 4.2, "y": 1}],
-                      "view": "Rice Rule"},
-                     {"disabled": None, "key": "bar",
-                      "values": [{"x": 2.7, "y": 2}, {"x": 4.2, "y": 1}],
-                      "view": "One Half"}]]
+                      "view": "Rice Rule"}]]
         self.assertEqual(expected, chart.render())
 
     @ddt.data(
@@ -252,8 +249,7 @@ class HistogramChartTestCase(test.TestCase):
                       {"bins": 2, "view": "Sturges Formula",
                        "x": [2.5, 4.0], "y": [0, 0]},
                       {"bins": 3, "view": "Rice Rule",
-                       "x": [2.0, 3.0, 4.0], "y": [0, 0, 0]},
-                      {"bins": 1, "view": "One Half", "x": [4.0], "y": [0]}]},
+                       "x": [2.0, 3.0, 4.0], "y": [0, 0, 0]}]},
         {"base_size": 100, "min_value": 27, "max_value": 42,
          "expected": [
              {"bins": 10, "view": "Square Root Choice",
@@ -264,14 +260,7 @@ class HistogramChartTestCase(test.TestCase):
                     42.0], "y": [0, 0, 0, 0, 0, 0, 0, 0]},
              {"bins": 10, "view": "Rice Rule",
               "x": [28.5, 30.0, 31.5, 33.0, 34.5, 36.0, 37.5, 39.0, 40.5,
-                    42.0], "y": [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]},
-             {"bins": 50, "view": "One Half",
-              "x": [27.3, 27.6, 27.9, 28.2, 28.5, 28.8, 29.1, 29.4, 29.7,
-                    30.0, 30.3, 30.6, 30.9, 31.2, 31.5, 31.8, 32.1, 32.4,
-                    32.7, 33.0, 33.3, 33.6, 33.9, 34.2, 34.5, 34.8, 35.1,
-                    35.4, 35.7, 36.0, 36.3, 36.6, 36.9, 37.2, 37.5, 37.8,
-                    38.1, 38.4, 38.7, 39.0, 39.3, 39.6, 39.9, 40.2, 40.5,
-                    40.8, 41.1, 41.4, 41.7, 42.0], "y": [0] * 50}]})
+                    42.0], "y": [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]}]})
     @ddt.unpack
     def test_views(self, base_size=None, min_value=None, max_value=None,
                    expected=None):
@@ -299,9 +288,7 @@ class MainHistogramChartTestCase(test.TestCase):
             {"disabled": None, "key": "task", "view": "Rice Rule",
              "values": [{"x": 3.666666666666667, "y": 3},
                         {"x": 5.333333333333334, "y": 0},
-                        {"x": 7.0, "y": 0}]},
-            {"disabled": None, "key": "task", "view": "One Half",
-             "values": [{"x": 4.5, "y": 3}, {"x": 7.0, "y": 0}]}]
+                        {"x": 7.0, "y": 0}]}]
         self.assertEqual([expected], chart.render())
 
 
@@ -324,9 +311,7 @@ class AtomicHistogramChartTestCase(test.TestCase):
                          {"x": 2.8, "y": 1}]},
              {"disabled": 0, "key": "foo", "view": "Rice Rule",
               "values": [{"x": 2.0, "y": 2}, {"x": 2.4, "y": 0},
-                         {"x": 2.8, "y": 1}]},
-             {"disabled": 0, "key": "foo", "view": "One Half",
-              "values": [{"x": 2.2, "y": 2}, {"x": 2.8, "y": 1}]}],
+                         {"x": 2.8, "y": 1}]}],
             [{"disabled": 1, "key": "bar", "view": "Square Root Choice",
               "values": [{"x": 4.3, "y": 2}, {"x": 5.5, "y": 1}]},
              {"disabled": 1, "key": "bar", "view": "Sturges Formula",
@@ -334,68 +319,114 @@ class AtomicHistogramChartTestCase(test.TestCase):
                          {"x": 5.5, "y": 1}]},
              {"disabled": 1, "key": "bar", "view": "Rice Rule",
               "values": [{"x": 3.9, "y": 2}, {"x": 4.7, "y": 0},
-                         {"x": 5.5, "y": 1}]},
-             {"disabled": 1, "key": "bar", "view": "One Half",
-              "values": [{"x": 4.3, "y": 2}, {"x": 5.5, "y": 1}]}]]
+                         {"x": 5.5, "y": 1}]}]]
         self.assertEqual(expected, chart.render())
 
 
-class TableTestCase(test.TestCase):
-
-    class Table(charts.Table):
-        columns = ["name", "foo", "bar"]
-        foo = mock.Mock()
-        bar = mock.Mock()
-
-        def _init_columns(self):
-            return costilius.OrderedDict(
-                [("foo", self.foo), ("bar", self.bar)])
-
-        def _map_iteration_values(self, iteration):
-            return [("value_" + k, iteration[k]) for k in ["a", "b"]]
-
-        def render(self):
-            return self._data
-
-    def setUp(self, *args, **kwargs):
-        super(TableTestCase, self).setUp(*args, **kwargs)
-        self.bench_info = {"iterations_count": 42,
-                           "atomic": {"a": {}, "b": {}, "c": {}}}
-
-    def test_add_iteration_and_render(self):
-        self.assertRaises(TypeError, charts.Table, self.bench_info)
-        table = self.Table(self.bench_info)
-        self.assertEqual(costilius.OrderedDict(), table.render())
-        [table.add_iteration({"a": i, "b": 43 - i}) for i in range(1, 43)]
-        self.assertEqual(
-            costilius.OrderedDict(
-                [("value_a", costilius.OrderedDict([("foo", table.foo),
-                                                    ("bar", table.bar)])),
-                 ("value_b", costilius.OrderedDict([("foo", table.foo),
-                                                    ("bar", table.bar)]))]),
-            table.render())
+MAIN_STATS_TABLE_COLUMNS = ["Action", "Min (sec)", "Median (sec)",
+                            "90%ile (sec)", "95%ile (sec)", "Max (sec)",
+                            "Avg (sec)", "Success", "Count"]
 
 
+def generate_iteration(duration, error, *args):
+    return {
+        "atomic_actions": costilius.OrderedDict(args),
+        "duration": duration,
+        "error": error
+    }
+
+
+@ddt.ddt
 class MainStatsTableTestCase(test.TestCase):
 
-    def setUp(self, *args, **kwargs):
-        super(MainStatsTableTestCase, self).setUp(*args, **kwargs)
-        self.bench_info = {"iterations_count": 42,
-                           "atomic": {"a": {}, "b": {}, "c": {}}}
-        self.columns = [
-            "Action", "Min (sec)", "Median (sec)", "90%ile (sec)",
-            "95%ile (sec)", "Max (sec)", "Avg (sec)", "Success", "Count"]
+    @ddt.data(
+        {
+            "info": {
+                "iterations_count": 1,
+                "atomic": costilius.OrderedDict([("foo", {}), ("bar", {})])
+            },
+            "data": [
+                generate_iteration(10.0, False, ("foo", 1.0), ("bar", 2.0))
+            ],
+            "expected": {
+                "cols": MAIN_STATS_TABLE_COLUMNS,
+                "rows": [
+                    ["foo", 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, "100.0%", 1],
+                    ["bar", 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, "100.0%", 1],
+                    ["total", 10.0, 10.0, 10.0, 10.0, 10.0, 10.0, "100.0%", 1],
+                ]
+            }
+        },
+        {
+            "info": {"iterations_count": 2, "atomic": {"foo": {}}},
+            "data": [
+                generate_iteration(10.0, True, ("foo", 1.0)),
+                generate_iteration(10.0, True, ("foo", 2.0))
+            ],
+            "expected": {
+                "cols": MAIN_STATS_TABLE_COLUMNS,
+                "rows": [
+                    ["foo", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a",
+                     2],
+                    ["total", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a",
+                     2],
+                ]
+            }
+        },
+        {
+            "info": {"iterations_count": 2, "atomic": {"foo": {}}},
+            "data": [
+                generate_iteration(10.0, False, ("foo", 1.0)),
+                generate_iteration(20.0, True, ("foo", 2.0))
+            ],
+            "expected": {
+                "cols": MAIN_STATS_TABLE_COLUMNS,
+                "rows": [
+                    ["foo", 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, "50.0%", 2],
+                    ["total", 10.0, 10.0, 10.0, 10.0, 10.0, 10.0, "50.0%", 2]
+                ]
+            }
+        },
+        {
+            "info": {
+                "iterations_count": 4,
+                "atomic": costilius.OrderedDict([("foo", {}), ("bar", {})])
+            },
+            "data": [
+                generate_iteration(10.0, False, ("foo", 1.0), ("bar", 4.0)),
+                generate_iteration(20.0, False, ("foo", 2.0), ("bar", 4.0)),
+                generate_iteration(30.0, False, ("foo", 3.0), ("bar", 4.0)),
+                generate_iteration(40.0, True, ("foo", 4.0), ("bar", 4.0))
+            ],
+            "expected": {
+                "cols": MAIN_STATS_TABLE_COLUMNS,
+                "rows": [
+                    ["foo", 1.0, 2.0, 2.8, 2.9, 3.0, 2.0, "75.0%", 4],
+                    ["bar", 4.0, 4.0, 4.0, 4.0, 4.0, 4.0, "75.0%", 4],
+                    ["total", 10.0, 20.0, 28.0, 29.0, 30.0, 20.0, "75.0%", 4]
+                ]
+            }
+        },
+        {
+            "info": {
+                "iterations_count": 0,
+                "atomic": costilius.OrderedDict()
+            },
+            "data": [],
+            "expected": {
+                "cols": MAIN_STATS_TABLE_COLUMNS,
+                "rows": [
+                    ["total", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a", "n/a",
+                     0]
+                ]
+            }
+        }
+    )
+    @ddt.unpack
+    def test_add_iteration_and_render(self, info, data, expected):
 
-    def test_add_iteration_and_render(self):
-        table = charts.MainStatsTable({"iterations_count": 42,
-                                       "atomic": {"foo": {}, "bar": {}}})
-        [table.add_iteration(
-            {"atomic_actions": costilius.OrderedDict([("foo", i),
-                                                      ("bar", 43 - 1)]),
-             "duration": i, "error": i % 40}) for i in range(1, 43)]
-        expected_rows = [
-            ["foo", 1.0, 21.5, 38.5, 40.5, 42.0, 21.5, "100.0%", 42.0],
-            ["bar", 42.0, 42.0, 42.0, 42.0, 42.0, 42.0, "100.0%", 42.0],
-            ["total", 0.0, 0.0, 0.0, 0.0, 40.0, 0.952, "100.0%", 42.0]]
-        self.assertEqual({"cols": self.columns, "rows": expected_rows},
-                         table.render())
+        table = charts.MainStatsTable(info)
+        for el in data:
+            table.add_iteration(el)
+
+        self.assertEqual(expected, table.render())
